sp500 <- read.table("sp500.dat")
gs <- read.table("goldmanSacks.dat")
ford <- read.table("ford.dat")

alpha2=0.025
alpha1=0.05
i1=as.integer(alpha1*500)
i2=as.integer(alpha2*500)

# Add name "Data" in Line 1 in sp500.dat, then
D <- read.table("sp500.dates")
t <- D[,1]
D <- strptime(t, "%Y-%m-%d") #convert to  POSIXlt (a date class)
D <- D[3:659]

sp500 <- 100*diff(log(sp500[,6])) # Now n=657
gs <- 100*diff(log(gs[,6]))
ford <- 100*diff(log(ford[,6]))

library(rugarch)
m1 = ugarchspec(variance.model=list(model="apARCH", garchOrder=c(1,1)), fixed.pars=list(delta=1),  mean.model=list(armaOrder=c(0,0), include.mean=F))

p=4*657
T <- matrix(1:p, ncol=4, nrow=657)

for(n in 503:656) { 
	T[(n+1),1]=sp500[n+1]  # value to be predicted
	X <- sp500[(n-500+1):n] # using the previous 500 data points
	Xres <- ugarchfit(m1, X)
	t <- coef(Xres)
	T[(n+1),2]=t[1]+t[2]*(abs(X[500])-t[4]*X[500])+t[3]*(sigma(Xres)[500])
 			# predicted SDT for T[(n+1),1]	
	Res <- X/sigma(Xres)
	Res <- sort(as.vector(Res))
	T[(n+1),3]=Res[i1] # alpha1-th quantile of residuals
	T[(n+1),4]=Res[i2] # alpha2-th quantile of residuals
}	

ps.options(horizontal=F, width=4.8, height=6, pointsize=9)
postscript("Fig315.eps", onefile=F, print=F)
par(mar=c(3,3,3,1),mfrow=c(3,1), mgp=c(1.8,0.6,0))
plot(D[504:657], sp500[504:657], type='l', xlab='', ylab='', ylim=c(-3.9,2.5), main='Daily returns of S&P500 index in 2012')
lines(D[504:657], T[504:657,2]*T[504:657,3], col="blue")
lines(D[504:657], T[504:657,2]*T[504:657,4], col="darkgreen")
lines(D[504:657], -T[504:657,2]*1.96, col="red")
lines(D[504:657], -T[504:657,2]*1.64, col="orange")
# print(table((T[504:657,2]*T[504:657,4])<T[504:657,1])/154)
# print(table((T[504:657,2]*T[504:657,3])<T[504:657,1])/154)
# print(table((-T[504:657,2]*1.64)<T[504:657,1])/154)
# print(table((-T[504:657,2]*1.96)<T[504:657,1])/154)


sp500 <- gs

for(n in 503:656) { 
	T[(n+1),1]=sp500[n+1]  # value to be predicted
	X <- sp500[(n-500+1):n] # using the previous 500 data points
	Xres <- ugarchfit(m1, X)
	t <- coef(Xres)
	T[(n+1),2]=t[1]+t[2]*(abs(X[500])-t[4]*X[500])+t[3]*(sigma(Xres)[500])
 			# predicted SDT for T[(n+1),1]	
	Res <- X/sigma(Xres)
	Res <- sort(as.vector(Res))
	T[(n+1),3]=Res[i1] # alpha1-th quantile of residuals
	T[(n+1),4]=Res[i2] # alpha2-th quantile of residuals
}	

plot(D[504:657], sp500[504:657], type='l', xlab='', ylab='', ylim=c(-7.5,6), main='Daily returns of Goldman Sacks stock in 2012')
lines(D[504:657], T[504:657,2]*T[504:657,3], col="blue")
lines(D[504:657], T[504:657,2]*T[504:657,4], col="darkgreen")
lines(D[504:657], -T[504:657,2]*1.96, col="red")
lines(D[504:657], -T[504:657,2]*1.64, col="orange")
print(table((T[504:657,2]*T[504:657,4])<T[504:657,1])/154)
print(table((T[504:657,2]*T[504:657,3])<T[504:657,1])/154)
print(table((-T[504:657,2]*1.64)<T[504:657,1])/154)
print(table((-T[504:657,2]*1.96)<T[504:657,1])/154)



sp500 <- ford


for(n in 503:656) { 
	T[(n+1),1]=sp500[n+1]  # value to be predicted
	X <- sp500[(n-500+1):n] # using the previous 500 data points
	Xres <- ugarchfit(m1, X)
	t <- coef(Xres)
	T[(n+1),2]=t[1]+t[2]*(abs(X[500])-t[4]*X[500])+t[3]*(sigma(Xres)[500])
 			# predicted SDT for T[(n+1),1]	
	Res <- X/sigma(Xres)
	Res <- sort(as.vector(Res))
	T[(n+1),3]=Res[i1] # alpha1-th quantile of residuals
	T[(n+1),4]=Res[i2] # alpha2-th quantile of residuals
}	


plot(D[504:657], sp500[504:657], type='l', xlab='', ylab='', ylim=c(-6.0,4), main='Daily returns of Ford stock in 2012')
lines(D[504:657], T[504:657,2]*T[504:657,3], col="blue")
lines(D[504:657], T[504:657,2]*T[504:657,4], col="darkgreen")
lines(D[504:657], -T[504:657,2]*1.96, col="red")
lines(D[504:657], -T[504:657,2]*1.64, col="orange")
print(table((T[504:657,2]*T[504:657,4])<T[504:657,1])/154)
print(table((T[504:657,2]*T[504:657,3])<T[504:657,1])/154)
print(table((-T[504:657,2]*1.64)<T[504:657,1])/154)
print(table((-T[504:657,2]*1.96)<T[504:657,1])/154)




dev.off()

